moBestPrices = {};
source(MultiOverlayV4.modDir.."scripte/module/bestPrices/moBestPricesConfig.lua");
--source(MultiOverlayV4.modDir.."scripte/module/bestPrices/moBestPricesOutput.lua");

function moBestPrices:update(dt)
	if moModule:getActiveRunningModul( {modName="multiOverlayV4", modulName="bestPrices"} ) then
		if moBestPricesConfig.values.updateTimer > 0 then
			if g_currentMission.hl.timers["updateBestPrices"] ~= nil then return;end;
			g_currentMission.hl.addTimer( {delay=moBestPricesConfig.values.updateTimer, name="updateBestPrices", repeatable=1, ms=moBestPricesConfig.values.msec, action=moBestPrices.load} ); 
		else
			moBestPrices:load();
		end;
	end;
end;

function moBestPrices.loadOnStart(update) --onStartMission Callback, update = updateByPlayer !new object or delete object etc. delete your old cmds!	
	if moBestPricesConfig.values == nil then moBestPricesConfig:defaultValues();end;	
	moBestPrices:loadStations();
	moBestPrices:loadAllBestPrices();	
end;

function moBestPrices.load() --updateTimer Callback	
	moBestPrices:loadAllBestPrices();	
end;

function moBestPrices:loadAllBestPrices()	
	local playerFarmId = g_currentMission.player.farmId;	
	--local farmId = g_currentMission:getFarmId()
	local bestPrice = {};	
	if moBestPricesConfig.values.sellingStations ~= nil and #moBestPricesConfig.values.sellingStations > 0 then
		for s=1, #moBestPricesConfig.values.sellingStations do
			if moBestPricesConfig.values.sellingStations[s] ~= nil then
				local sellingStation = moBestPricesConfig.values.sellingStations[s];				
				local node = Utils.getNoNil(sellingStation.rootNode, sellingStation.nodeId);					
				local stationName = Utils.getNoNil(sellingStation.uiName, sellingStation.stationName);
				for fillType,isAccepted in pairs(sellingStation:getSupportedFillTypes()) do
					if isAccepted then
						local difficultyMultiplier = 1000;
						if bestPrice[fillType] == nil then
							bestPrice[fillType] = {};									
						end;								
						local price = math.floor(sellingStation:getEffectiveFillTypePrice(fillType) * difficultyMultiplier - 0.5);
						local priceTrend = sellingStation:getCurrentPricingTrend(fillType)
						local isPriceTrend = "normal";
						if priceTrend ~= nil then
							if Utils.isBitSet(priceTrend, SellingStation.PRICE_GREAT_DEMAND) then
								isPriceTrend = "greatDemand";								
							elseif Utils.isBitSet(priceTrend, SellingStation.PRICE_CLIMBING) then
								isPriceTrend = "up";
							elseif Utils.isBitSet(priceTrend, SellingStation.PRICE_FALLING) then
								isPriceTrend = "down";
							end;
						end;							
						bestPrice[fillType][#bestPrice[fillType]+1] = {price, stationName, isPriceTrend, index, node};
					end;
				end;				
			end;
		end;
	end;
	function comp(w1,w2)		
		if w1[1] > w2[1] then
			return true;
		end;				
	end;		
	for index, _ in pairs(bestPrice) do
		table.sort(bestPrice[index], comp);
	end;
	moFillTypes.values.bestPrice = bestPrice; --new
end;

function moBestPrices:loadStations()
	moBestPricesConfig.values.sellingStations = {};
	moBestPricesConfig.values.buyingStations = {};
	for _, station in pairs(g_currentMission.storageSystem:getUnloadingStations()) do
		if station:isa(SellingStation) and not station.hideFromPricesMenu then
			station.uiName = station:getName();
			table.insert(moBestPricesConfig.values.sellingStations, station);
		end;
	end;
	for _, station in pairs(g_currentMission.storageSystem:getLoadingStations()) do
		if station:isa(BuyingStation) then
			station.uiName = station:getName();
			table.insert(moBestPricesConfig.values.buyingStations, station);
		end;
	end;
	function comp(w1,w2)			
		if w1.uiName < w2.uiName then
			return true;
		end;			
	end;
	table.sort(moBestPricesConfig.values.sellingStations, comp);
	table.sort(moBestPricesConfig.values.buyingStations, comp);
end;


moModule:addActiveRunningModul( {modName="multiOverlayV4", modulName="bestPrices" } );
moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="onStartMission", callback=moBestPrices.loadOnStart } );
moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="update", callback=moBestPrices.update } );