moFields = {};
source(MultiOverlayV4.modDir.."scripte/module/fields/moFieldsConfig.lua");
source(MultiOverlayV4.modDir.."scripte/module/fields/moFieldsOutput.lua");

function moFields:update(dt)
	if moModule:getActiveRunningModul( {modName="multiOverlayV4", modulName="fields"} ) then
		if moFields.values.updateTimer > 0 then
			if g_currentMission.hl.timers["updateFields"] ~= nil then return;end;
			g_currentMission.hl.addTimer( {delay=moFields.values.updateTimer, name="updateFields", repeatable=1, ms=moFields.values.msec, action=moFields.load} ); 
		else
			moFields:load();
		end;
	end;
end;

function moFields.loadOnStart(update) --onStartMission Callback, update = updateByPlayer !new object or delete obbject etc. delete your old cmds!	
	if moFields.values == nil then moFieldsConfig:defaultValues();end;	
	if update == nil or not update then moFieldsConfig:generateCmd();end;
	if update == nil or update then moFields:loadAllFields();moFields:setFieldsData();end;
end;

function moFields.load() --updateTimer Callback	
	--moFields:loadAllFields();
	moFields:setFieldsData();
end;

function moFields:loadAllFields()	
	local playerFarmId = g_currentMission.player.farmId;	
	moFields.values.fields = {}; --new
	for fieldId, field in pairs (g_fieldManager.fields) do
		if field.farmland ~= nil and g_farmlandManager:getFarmlandOwner(field.farmland.id) == playerFarmId then
			local field_ = {
				farmlandId = field.farmland.id;
				fieldId = field.fieldId;
				fieldHectar = Utils.getNoNil(field.fieldArea, 0);				
				fieldHectarString = tostring(string.format("%1.2f", "".. tostring(Utils.getNoNil(field.fieldArea, 0)))).. " ".. tostring(g_i18n:getText("unit_haShort"));
				fieldData = {};
			};
			table.insert(moFields.values.fields, field_);
		end;
	end;	
end;

function moFields:setFieldsData()
	local playerFarmId = g_currentMission.player.farmId;
	if moFields.values.fields ~= nil and #moFields.values.fields > 0 then
		for a=1, #moFields.values.fields do
			if moFields.values.fields[a] ~= nil then
				local field = g_fieldManager:getFieldByIndex(moFields.values.fields[a].fieldId);
				if field ~= nil then					
					moFields.values.fields[a].fieldData = {}; --new
					local sprayFactor, plowFactor, limeFactor, weedFactor, maxWeedState, stubbleFactor, rollerFactor = moFields:getFieldData(field);
					moFields.values.fields[a].fieldData.fruitType = field.fruitType;
					moFields.values.fields[a].fieldData.growth = 0;
					local fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(field.fruitType);
					if fruitDesc ~= nil then 
						local fillTypeFruitIndex = g_fruitTypeManager:getFillTypeIndexByFruitTypeIndex(fruitDesc.index);
						if fillTypeFruitIndex ~= nil and fillTypeFruitIndex > 0 and moFillTypes.values.fillTypes[fillTypeFruitIndex] ~= nil then
							moFields.values.fields[a].fieldData.fruitTypeName = moFillTypes.values.fillTypes[fillTypeFruitIndex].name;
						else
							moFields.values.fields[a].fieldData.fruitTypeName = Utils.getNoNil(fruitDesc.fillType.title, "Unknown");
						end;
						moFields.values.fields[a].fieldData.maxGrowthState = Utils.getNoNil(FieldUtil.getMaxGrowthState(field, fruitDesc.index), 0);
						moFields.values.fields[a].fieldData.maxHarvestState = FieldUtil.getMaxHarvestState(field, fruitDesc.index);
						local witheredState = fruitDesc.maxHarvestingGrowthState + 1;
						--if fruitDesc.maxPreparingGrowthState >= 0 then
						--	witheredState = fruitDesc.maxPreparingGrowthState + 1;
						--end;
						local maxGrowingState = fruitDesc.minHarvestingGrowthState - 1;
						if fruitDesc.minPreparingGrowthState >= 0 then
							maxGrowingState = math.min(maxGrowingState, fruitDesc.minPreparingGrowthState - 1);
						end
						if maxGrowingState >= witheredState then
							moFields.values.fields[a].fieldData.withered = true;
						end;
						if moFields.values.fields[a].fieldData.maxHarvestState ~= nil then
							witheredState = fruitDesc.maxHarvestingGrowthState + 1;
							if moFields.values.fields[a].fieldData.maxHarvestState >= witheredState then
								moFields.values.fields[a].fieldData.withered = true;
							end;
						end;
						local growthTxt, sortByGrowth, color = moFields:getFruitTypeStateText(fruitDesc.index, moFields.values.fields[a].fieldData.maxGrowthState, witheredState, moFields.values.fields[a].fieldData.maxHarvestState);
						moFields.values.fields[a].fieldData.growthTxt = growthTxt;
						moFields.values.fields[a].fieldData.growth = sortByGrowth;
						moFields.values.fields[a].fieldData.growthTxtColor = color;
						moFields.values.fields[a].fieldData.growthTxtNew = moFields:getFruitTypeStateTextNew(fruitDesc.index, moFields.values.fields[a].fieldData.maxGrowthState, witheredState, moFields.values.fields[a].fieldData.maxHarvestState);
					end;
					if moFields.values.fields[a].fieldData.withered ~= nil then
						moFields.values.fields[a].fieldData.growthTxtColor = "orangeRed";
					else
						moFields.values.fields[a].fieldData.withered = false;
					end;
					moFields.values.fields[a].fieldData.sprayFactor = sprayFactor;
					moFields.values.fields[a].fieldData.sprayProzent = Utils.getNoNil(string.format("%d %%", sprayFactor * 100), "0%");
					moFields.values.fields[a].fieldData.plowFactor = plowFactor;
					moFields.values.fields[a].fieldData.plowProzent = Utils.getNoNil(string.format("%d %%", plowFactor * 100), "0");
					moFields.values.fields[a].fieldData.limeFactor = limeFactor;
					moFields.values.fields[a].fieldData.limeProzent = Utils.getNoNil(string.format("%d %%", limeFactor * 100), "0%");
					moFields.values.fields[a].fieldData.weedFactor = weedFactor;
					moFields.values.fields[a].fieldData.weedProzent = Utils.getNoNil(string.format("%d %%", 100-(weedFactor * 100)), "0%");
					--moFields.values.fields[a].fieldData.weed = 100-(Utils.getNoNil(weedFactor*100, 0));
					moFields.values.fields[a].fieldData.maxWeedState = maxWeedState;
					
					moFields.values.fields[a].fieldData.stubbleFactor = stubbleFactor;
					moFields.values.fields[a].fieldData.stubbleProzent = Utils.getNoNil(string.format("%d %%", stubbleFactor * 100), "0%");
					moFields.values.fields[a].fieldData.rollerFactor = rollerFactor;
					moFields.values.fields[a].fieldData.rollerProzent = Utils.getNoNil(string.format("%d %%", rollerFactor * 100), "0%");
					
				end;
			end;
		end;
		function fields(w1,w2)			
			if w1.fieldId < w2.fieldId then
				return true;
			end;			
		end;
		function fruitName(w1,w2)			
			if w1.fieldData == nil or w2.fieldData == nil or w1.fieldData.fruitTypeName == nil or w2.fieldData.fruitTypeName == nil then return false;end;
			if w1.fieldData.fruitTypeName < w2.fieldData.fruitTypeName then
				return true;
			end;			
		end;
		function state(w1,w2)			
			if w1.fieldData.growth > w2.fieldData.growth then
				return true;
			end;			
		end;
		function sprayFactor(w1,w2)			
			if w1.fieldData == nil or w2.fieldData == nil or w1.fieldData.sprayFactor == nil or w2.fieldData.sprayFactor == nil then return false;end;
			if w1.fieldData.sprayFactor > w2.fieldData.sprayFactor then
				return true;
			end;			
		end;
		function limeFactor(w1,w2)			
			if w1.fieldData == nil or w2.fieldData == nil or w1.fieldData.limeFactor == nil or w2.fieldData.limeFactor == nil then return false;end;
			if w1.fieldData.limeFactor > w2.fieldData.limeFactor then
				return true;
			end;			
		end;
		function weedFactor(w1,w2)			
			if w1.fieldData == nil or w2.fieldData == nil or w1.fieldData.weedFactor == nil or w2.fieldData.weedFactor == nil then return false;end;
			if w1.fieldData.weedFactor > w2.fieldData.weedFactor then
				return true;
			end;			
		end;
		function rollerFactor(w1,w2)			
			if w1.fieldData == nil or w2.fieldData == nil or w1.fieldData.rollerFactor == nil or w2.fieldData.rollerFactor == nil then return false;end;
			if w1.fieldData.rollerFactor > w2.fieldData.rollerFactor then
				return true;
			end;			
		end;
		function plowFactor(w1,w2)			
			if w1.fieldData == nil or w2.fieldData == nil or w1.fieldData.plowFactor == nil or w2.fieldData.plowFactor == nil then return false;end;
			if w1.fieldData.plowFactor > w2.fieldData.plowFactor then
				return true;
			end;			
		end;
		if moFields.values.sortBy == 1 then
			table.sort(moFields.values.fields, fields);
		elseif moFields.values.sortBy == 2 then
			table.sort(moFields.values.fields, fruitName);	
		elseif moFields.values.sortBy == 3 then
			table.sort(moFields.values.fields, state);
		elseif moFields.values.sortBy == 4 then
			table.sort(moFields.values.fields, sprayFactor);	
		elseif moFields.values.sortBy == 5 then
			table.sort(moFields.values.fields, limeFactor);		
		elseif moFields.values.sortBy == 6 then
			table.sort(moFields.values.fields, weedFactor);	
		elseif moFields.values.sortBy == 7 then
			table.sort(moFields.values.fields, rollerFactor);	
		else
			table.sort(moFields.values.fields, plowFactor);
		end;
	end;
end;

function moFields:getFieldData(field)
	local sprayFactor = FieldUtil.getSprayFactor(field);
	local plowFactor = 0;
	if g_currentMission.missionInfo.plowingRequiredEnabled then plowFactor = FieldUtil.getPlowFactor(field);end;
	local limeFactor = 0;
	if g_currentMission.missionInfo.limeRequired then limeFactor = FieldUtil.getLimeFactor(field);end;
	local weedFactor = 0;
	local maxWeedState = 0;
	if g_currentMission.missionInfo.weedsEnabled then weedFactor = FieldUtil.getWeedFactor(field);maxWeedState = FieldUtil.getMaxWeedState(field);end;
	local stubbleFactor = FieldUtil.getStubbleFactor(field);
	local rollerFactor = 0
	if g_currentMission.missionInfo.stonesEnabled then rollerFactor = FieldUtil.getRollerFactor(field);end;
	return sprayFactor, plowFactor, limeFactor, weedFactor, maxWeedState, stubbleFactor, rollerFactor;
end;

function moFields:getFruitTypeStateText(fruitTypeIndex, fruitGrowthState, witheredState, fruitHarvestState)	
	local text = "";
	local color = nil;
	local sortByGrowth = -1;
	if fruitTypeIndex > 0 then --and (g_seasons == nil or MultiOverlayV3.otherMods.seasons == nil) then
		local fruitType = g_fruitTypeManager:getFruitTypeByIndex(fruitTypeIndex);		
		local maxGrowingState = fruitType.minHarvestingGrowthState-1;
		if fruitType.minPreparingGrowthState >= 0 then
			maxGrowingState = math.min(maxGrowingState, fruitType.minPreparingGrowthState-1);
		end;
		if fruitGrowthState == fruitType.cutState then
			text = g_i18n:getText("ui_growthMapCut");			 
			sortByGrowth = 0;
		elseif fruitGrowthState == witheredState then
			text = g_i18n:getText("ui_growthMapWithered");
			color = "red";
			sortByGrowth = -2;
		elseif fruitGrowthState > 0 and fruitGrowthState <= maxGrowingState then
			--text = g_i18n:getText("ui_growthMapGrowing")
			local prozent = g_i18n:formatNumber((100/(maxGrowingState+1)+0.5)*(fruitGrowthState-1));
			text = tostring(prozent).. "%";
			sortByGrowth = (100/(maxGrowingState+1)+0.5)*(fruitGrowthState-1);
		elseif fruitType.minPreparingGrowthState >= 0 and fruitType.minPreparingGrowthState <= fruitGrowthState and fruitGrowthState <= fruitType.maxPreparingGrowthState then
			text = g_i18n:getText("ui_growthMapReadyToPrepareForHarvest");
			color = "yellow";
			sortByGrowth = 100;
		elseif fruitGrowthState >= fruitType.minHarvestingGrowthState and fruitGrowthState <= fruitType.maxHarvestingGrowthState then
			text = g_i18n:getText("ui_growthMapReadyToHarvest");
			color = "yellow"
			sortByGrowth = 100;
		end;
		if string.len(text) == 0 and fruitHarvestState ~= nil then
			if fruitHarvestState <= fruitType.maxHarvestingGrowthState then text = g_i18n:getText("ui_growthMapReadyToHarvest");color="yellow";sortByGrowth=100;end;
			if fruitHarvestState > fruitType.maxHarvestingGrowthState then text = g_i18n:getText("ui_growthMapWithered");color="red";sortByGrowth=-2;end;
		end;
		if string.len(text) == 0 and fruitHarvestState == nil and fruitGrowthState == 0 then
			text = g_i18n:getText("ui_growthMapCut");
			sortByGrowth = 0;
		end;	
	end;	
	return text, sortByGrowth, color;
end;

function moFields:getFruitTypeStateTextNew(fruitTypeIndex, fruitGrowthState, witheredState, fruitHarvestState)
	local text = "";
	if fruitTypeIndex > 0 then
		local fruitType = g_fruitTypeManager:getFruitTypeByIndex(fruitTypeIndex);
		local maxGrowingState = fruitType.minHarvestingGrowthState - 1;
		if fruitType.minPreparingGrowthState >= 0 then
			maxGrowingState = math.min(maxGrowingState, fruitType.minPreparingGrowthState - 1);
		end;		
		local isGrowing = false;
		--if fruitGrowthState > 0 then
			if fruitGrowthState == fruitType.cutState then
				text = g_i18n:getText("ui_growthMapCut");
			elseif fruitGrowthState == witheredState then
				text = g_i18n:getText("ui_growthMapWithered");
			elseif fruitGrowthState > 0 and fruitGrowthState <= maxGrowingState then
				text = g_i18n:getText("ui_growthMapGrowing");
				isGrowing = true;
			elseif fruitType.minPreparingGrowthState >= 0 and fruitType.minPreparingGrowthState <= fruitGrowthState and fruitGrowthState <= fruitType.maxPreparingGrowthState then
				text = g_i18n:getText("ui_growthMapReadyToPrepareForHarvest");
				isGrowing = true;
			elseif fruitType.minHarvestingGrowthState <= fruitGrowthState and fruitGrowthState <= fruitType.maxHarvestingGrowthState then
				text = g_i18n:getText("ui_growthMapReadyToHarvest");
				isGrowing = true;
			end;			
		--end;
		if string.len(text) == 0 and fruitHarvestState ~= nil then
			if fruitHarvestState <= fruitType.maxHarvestingGrowthState then text = g_i18n:getText("ui_growthMapReadyToHarvest");end;
			if fruitHarvestState >= witheredState then text = g_i18n:getText("ui_growthMapWithered");end;
		end;
	end;
	return text;
end;

moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="onStartMission", callback=moFields.loadOnStart } );
moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="update", callback=moFields.update } );