moKeySettings = {};
source(MultiOverlayV4.modDir.."scripte/module/keySettings/moKeySettingsConfig.lua");
source(MultiOverlayV4.modDir.."scripte/module/keySettings/moKeySettingsOutput.lua");

function moKeySettings:update(dt)
	if moModule:getActiveRunningModul( {modName="multiOverlayV4", modulName="keySettings"} ) then
		if moKeySettings.values.updateTimer > 0 then
			if g_currentMission.hl.timers["updateKeySettings"] ~= nil then return;end;
			g_currentMission.hl.addTimer( {delay=moKeySettings.values.updateTimer, name="updateKeySettings", repeatable=1, ms=moKeySettings.values.msec, action=moKeySettings.load} ); 
		else
			moKeySettings:load();
		end;
	end;	
end;

function moKeySettings:updateTick(dt)
	if moKeySettings.values.disabledId ~= nil then
		for eventId, _ in pairs (moKeySettings.values.disabledId) do
			if g_inputBinding.events[eventId] ~= nil then g_inputBinding:setActionEventTextVisibility(eventId, false);end;
		end;
	end;
end;

function moKeySettings.loadOnStart(update) --onStartMission Callback, update = updateByPlayer !new object or delete obbject etc. delete your old cmds!	
	if moKeySettings.values == nil then moKeySettingsConfig:defaultValues();end;	
	if update == nil or not update then moKeySettingsConfig:generateCmd();end;
	if update == nil then moKeySettings:loadAllKeySettings(true);end;	
end;

function moKeySettings.load() --updateTimer Callback	
	moKeySettings:loadAllKeySettings();	
end;

function moKeySettings:loadAllKeySettings(generateCmd)	
	local playerFarmId = g_currentMission.player.farmId;	
	moKeySettings.values.modBindings = {}; --new	
	
	function setBindings(modName, action, ls)
		if moKeySettings.values.mod[modName] == nil then moKeySettings.values.mod[modName] = {viewEvents=false,name=modName,isReady=true,events={}};end;			
		if moKeySettings.values.modBindings[modName] == nil then moKeySettings.values.modBindings[modName] = {};end;
		local inputSymbol = string.format("input_%s", action.name);
		local modPart = modName and string.format(" in mod '%s'", modName) or "";
		if not ls then
			action.displayCategory = g_modManager:getModByName(modName).title;		
			action.displayNamePositive = action.name;
			action.displayNameNegative = action.name;
			local isComboAction = InputBinding.MOUSE_COMBOS[action.name] ~= nil and InputBinding.GAMEPAD_COMBOS[action.name] ~= nil;
			local isLocked = action.isLocked;
			local needLocalization = not isComboAction and not isLocked;
			local i18n = _G[modName].g_i18n;
			if i18n == nil then i18n = g_i18n;end;
			if action.axisType == InputAction.AXIS_TYPE.FULL then
				local symbolPositive = inputSymbol .. InputBinding.SYMBOL_AFFIX_POSITIVE;
				local symbolNegative = inputSymbol .. InputBinding.SYMBOL_AFFIX_NEGATIVE;							
				if i18n:hasText(symbolPositive) then
					action.displayNamePositive = i18n:getText(symbolPositive);
				elseif needLocalization then
					action.displayNamePositive = "Missing l10n";
				end;
				if i18n:hasText(symbolNegative) then
					action.displayNameNegative = i18n:getText(symbolNegative)
				elseif needLocalization then
					action.displayNameNegative = "Missing l10n";
				end;
			elseif i18n:hasText(inputSymbol) then
				action.displayNamePositive = i18n:getText(inputSymbol);
			elseif needLocalization then
				action.displayNamePositive = "Missing l10n";							
			end;
		end;		
		moKeySettings.values.modBindings[modName][#moKeySettings.values.modBindings[modName]+1] = {};						
		for k, v in pairs(action) do
			moKeySettings.values.modBindings[modName][#moKeySettings.values.modBindings[modName]][k] = v;							
		end;
		local nameAction = g_inputBinding.nameActions[moKeySettings.values.modBindings[modName][#moKeySettings.values.modBindings[modName]].name];
		if nameAction ~= nil then
			if g_inputBinding.actionEvents[nameAction] ~= nil then								
				local activeBindings = nameAction:getActiveBindings();
				if activeBindings ~= nil and #activeBindings > 0 then
					moKeySettings.values.modBindings[modName][#moKeySettings.values.modBindings[modName]].activeBindings = activeBindings;
					moKeySettings.values.modBindings[modName][#moKeySettings.values.modBindings[modName]].activeBindingsDisplayText = moKeySettings:getBindingInputDisplayText(activeBindings[1]);
				end;
				local bindings = nameAction:getBindings();
				if bindings ~= nil and #bindings > 0 then
					moKeySettings.values.modBindings[modName][#moKeySettings.values.modBindings[modName]].bindings = bindings;
					moKeySettings.values.modBindings[modName][#moKeySettings.values.modBindings[modName]].bindingsDisplayText = moKeySettings:getBindingInputDisplayText(bindings[1]);
				end;
				for _, event in ipairs(g_inputBinding.actionEvents[nameAction]) do									
					if event ~= nil and event.id ~= nil then
						moKeySettings.values.modBindings[modName][#moKeySettings.values.modBindings[modName]].event = {id=event.id,displayIsVisible=event.displayIsVisible,displayPriority=event.displayPriority,isActive=event.isActive};
						if moKeySettings.values.mod[modName].events[event.id] == nil then  
							moKeySettings.values.mod[modName].events[event.id] = {id=event.id,displayIsVisible=event.displayIsVisible,displayPriority=event.displayPriority,isActive=event.isActive};
						end;
					end;									
				end;
			end;
		end;
	end;
	
	for _, modDesc in ipairs(g_modManager:getMods()) do
		local xmlFile = loadXMLFile("ModFile", modDesc.modFile);		
		if modDesc.modName and g_modIsLoaded[tostring(modDesc.modName)] then
			local actionIndex = 0;						
			while true do
				local actionPath = "modDesc.actions".. string.format(".action(%d)", actionIndex);
				if not hasXMLProperty(xmlFile, actionPath) then
					break;
				end;
				local action = InputAction.createFromXML(xmlFile, actionPath);
				if action ~= nil then					
					setBindings(modDesc.modName, action, false);									
				end;
				actionIndex = actionIndex+1;
			end;			
		end;			
		delete(xmlFile);
	end;	
	local action = g_inputBinding.nameActions["TOGGLE_LIGHTS_FPS"];
	if action ~= nil and g_inputBinding.actionEvents[action] ~= nil then
		setBindings("Ls22-Default/OnFoot", action, true);		
	end;	
end;

function moKeySettings:getBindingInputDisplayText(binding)
	if #binding.axisNames < 1 then
		return "";
	end;
	local texts = {};
	local deviceLabel = "";
	local isKeyboard = binding.deviceId == InputDevice.DEFAULT_DEVICE_NAMES.KB_MOUSE_DEFAULT;
	if isKeyboard then
		for _, axis in pairs(binding.axisNames) do
			if InputBinding.MOUSE_BUTTONS[axis] then
				table.insert(texts, getMouseButtonName(Input[axis]));
			elseif InputBinding.MOUSE_AXES[axis] then
				table.insert(texts, moKeySettings:getMouseAxisDisplayText(axis));
			else
				table.insert(texts, KeyboardHelper.getDisplayKeyName(Input[axis]));
			end;
		end;
	else
		for _, axis in pairs(binding.axisNames) do
			if Input.buttonIdNameToId[axis] then
				table.insert(texts, moKeySettings:getGamepadButtonDisplayText(axis, binding.internalDeviceId));
			elseif Input.axisIdNameToId[axis] then
				table.insert(texts, moKeySettings:getGamepadAxisDisplayText(axis, binding.internalDeviceId));
			end;
		end;
		if getNumOfGamepads() > 1 or g_inputBinding:getHasMissingDevices() then
			local device = g_inputBinding:getDeviceByInternalId(binding.internalDeviceId);
			if device == nil then
				device = g_inputBinding:getMissingDeviceById(binding.deviceId);
			end;
			if device ~= nil then
				local gamepadName = device.deviceName;
				if g_i18n:hasText(gamepadName) then
					gamepadName = g_i18n:getText(gamepadName);
				end;
				deviceLabel = string.format(" [%s]", gamepadName);
			end;
		end;
	end;
	return table.concat(texts, " + ") .. deviceLabel;
end;

function moKeySettings:getMouseAxisDisplayText(axis)
	local text = "";
	if InputBinding.MOUSE_AXES[axis] == Input.AXIS_X then
		text = ControlsController.AXIS_NAME_X;
	elseif InputBinding.MOUSE_AXES[axis] == Input.AXIS_Y then
		text = ControlsController.AXIS_NAME_Y;
	end;
	if text ~= "" then
		local postFix = ControlsController.AXIS_AFFIX_POSITIVE;
		if axis:sub(axis:len()) == "-" then
			postFix = ControlsController.AXIS_AFFIX_NEGATIVE;
		end;
		text = text .. postFix;
	end;
	return text;
end;

function moKeySettings:getGamepadButtonDisplayText(buttonName, internalDeviceId)
	if internalDeviceId == nil or internalDeviceId < 0 then
		return string.format("%d", Input[buttonName] + 1);
	end;
	return getGamepadButtonLabel(Input[buttonName], internalDeviceId);
end;

function moKeySettings:getGamepadAxisDisplayText(axisName, internalDeviceId)
	local axisLabel = nil;
	if internalDeviceId == nil or internalDeviceId < 0 then
		axisLabel = string.format("Axis %d", Input[axisName] + 1);
	else
		axisLabel = getGamepadAxisLabel(Input[axisName], internalDeviceId);
	end;
	local directionAffix = ControlsController.AXIS_AFFIX_POSITIVE;
	if axisName:sub(axisName:len()) == "-" then
		directionAffix = ControlsController.AXIS_AFFIX_NEGATIVE;
	end;
	axisLabel = axisLabel .. directionAffix;
	return axisLabel;
end;

function moKeySettings:hashEvents(mod)
	local events = 0;
	if moKeySettings.values.modBindings[mod] ~= nil then
		for a=1, #moKeySettings.values.modBindings[mod] do
			if moKeySettings.values.modBindings[mod][a].event ~= nil then events = events+1;end;
		end;
	end;
	return events > 0,events;
end;

function moKeySettings:displayIsVisible(mod)	
	if moKeySettings.values.mod[mod] ~= nil and moKeySettings.values.mod[mod].isReady then
		for a=1, #moKeySettings.values.modBindings[mod] do
			if moKeySettings.values.modBindings[mod][a].event ~= nil and moKeySettings.values.modBindings[mod][a].event.id ~= nil then
				local eventId = moKeySettings.values.modBindings[mod][a].event.id;
				local eventStatus = moKeySettings.values.modBindings[mod][a].event.displayIsVisible;
				if g_inputBinding.events[eventId] ~= nil and eventStatus then 
					g_inputBinding:setActionEventTextVisibility(eventId, false);
					moKeySettings.values.disabledId[eventId] = true;
				end;
			end;
		end;
		moKeySettings.values.mod[mod].isReady = false;
	elseif moKeySettings.values.mod[mod] ~= nil and not moKeySettings.values.mod[mod].isReady then
		for a=1, #moKeySettings.values.modBindings[mod] do
			if moKeySettings.values.modBindings[mod][a].event ~= nil and moKeySettings.values.modBindings[mod][a].event.id ~= nil then
				local eventId = moKeySettings.values.modBindings[mod][a].event.id;
				local eventStatus = moKeySettings.values.mod[mod].events[eventId].displayIsVisible;
				if g_inputBinding.events[eventId] ~= nil and eventStatus then					
					g_inputBinding:setActionEventTextVisibility(eventId, eventStatus);					
					moKeySettings.values.disabledId[eventId] = nil;
				end;
			end;
		end;
		moKeySettings.values.mod[mod].isReady = true;
	end;
end;

function moKeySettings.setActionEventTextVisibility(eventId, isVisible)
	if moKeySettings.values ~= nil and moKeySettings.values.disabledId[eventId] ~= nil and moKeySettings.values.disabledId[eventId] then return;end;
	local event = g_inputBinding.events[eventId];
	local hasChange = false;
	if event then
		hasChange = event.displayIsVisible ~= isVisible;
		event.displayIsVisible = isVisible;
	end;
	if hasChange then
		g_inputBinding:refreshEventCollections();
	end;
end;
--InputBinding.setActionEventTextVisibility = Utils.overwrittenFunction(InputBinding.setActionEventTextVisibility, moKeySettings.setActionEventTextVisibility);
moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="onStartMission", callback=moKeySettings.loadOnStart } );
moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="update", callback=moKeySettings.update } );
moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="updateTick", callback=moKeySettings.updateTick } );