moMissions = {};
source(MultiOverlayV4.modDir.."scripte/module/missions/moMissionsConfig.lua");
source(MultiOverlayV4.modDir.."scripte/module/missions/moMissionsOutput.lua");

function moMissions:update(dt)
	if moModule:getActiveRunningModul( {modName="multiOverlayV4", modulName="missions"} ) then
		if moMissions.values.updateTimer > 0 then
			if g_currentMission.hl.timers["updateMissions"] ~= nil then return;end;
			g_currentMission.hl.addTimer( {delay=moMissions.values.updateTimer, name="updateMissions", repeatable=1, ms=moMissions.values.msec, action=moMissions.load} ); 
		else
			moMissions:load();
		end;
	end;
end;

function moMissions.loadOnStart(update) --onStartMission Callback, update = updateByPlayer !new object or delete obbject etc. delete your old cmds!	
	if moMissions.values == nil then moMissionsConfig:defaultValues();end;	
	if update == nil then moMissions:loadAllMissions(true);end;	
end;

function moMissions.load() --updateTimer Callback	
	moMissions:loadAllMissions();	
end;

function moMissions:loadAllMissions(generateCmd)	
	local playerFarmId = g_currentMission.player.farmId;	
	moMissions.values.missions = {}; --new
	local missionsList = g_missionManager:getMissionsList(playerFarmId);
	for _, mission in ipairs(missionsList) do
		local isActive = mission.status == AbstractMission.STATUS_RUNNING;
		local isFinished = mission.status == AbstractMission.STATUS_FINISHED;
		local isPossible = mission.status == AbstractMission.STATUS_STOPPED;
		local missionInfo = mission:getData()
		local contract = {
			mission = mission;
			active = isActive;
			finished = isFinished;
			possible = isPossible;
			jobType = missionInfo.jobType;
			prozent = g_i18n:formatNumber(Utils.getNoNil(mission.completion, 0)*100);
			isRunning = mission.isInMissionMap or (mission.activeMissionId ~= nil);
			numObjects = Utils.getNoNil(mission.numObjects, 0);
			reward = mission.reward;
			missionId = Utils.getNoNil(mission.id, 0);
		};
		if mission.sellPoint ~= nil then			
			contract.sellPointId = Utils.getNoNil(mission.sellPoint.rootNode, 0);			
		end;
		if mission.field ~= nil then
			contract.isField = true;
			contract.fieldId = mission.field.fieldId;
			contract.fieldHectar = mission.field.fieldArea;
			contract.fillType = mission.fillType;
			contract.fruitType = mission.fruitType;
		else
			contract.isField = false;
			contract.fieldId = 0;
			contract.fieldHectar = 0;
		end;
		if mission.vehicles ~= nil then
			contract.vehicle = Utils.getNoNil(mission.spawnedVehicles, false);
		else
			contract.vehicle = false;
		end;
		table.insert(moMissions.values.missions, contract);		
	end;	
	if generateCmd ~= nil and generateCmd then moMissionsConfig:generateCmd();end;
	function reward(w1,w2)			
		if w1.reward > w2.reward then
			return true;
		end;			
	end;
	function jobType(w1,w2)			
		if w1.jobType < w2.jobType then
			return true;
		end;			
	end;
	function hectar(w1,w2)			
		if w1.fieldHectar > w2.fieldHectar then
			return true;
		end;			
	end;
	function fieldId(w1,w2)			
		if w1.fieldId > w2.fieldId then
			return true;
		end;			
	end;	
	if moMissions.values.sortBy == 1 then
		table.sort(moMissions.values.missions, reward);
	elseif moMissions.values.sortBy == 2 then
		table.sort(moMissions.values.missions, jobType);
	elseif moMissions.values.sortBy == 3 then
		table.sort(moMissions.values.missions, hectar);
	elseif moMissions.values.sortBy == 4 then
		table.sort(moMissions.values.missions, fieldId);		
	end;	
end;

function moMissions:getInfoBar()
	local playerFarmId = g_currentMission.player.farmId;
	local missionsList = g_missionManager:getMissionsList(playerFarmId);
	local isNotice = false;
	local prozent = 0;
	local found = 0;
	for _, mission in ipairs(missionsList) do
		if mission.status == AbstractMission.STATUS_RUNNING or mission.status == AbstractMission.STATUS_FINISHED then
			if not isNotice and (mission.status == AbstractMission.STATUS_FINISHED or mission.status == AbstractMission.STATUS_STOPPED) then isNotice = true;end;
			prozent = prozent + (g_i18n:formatNumber(Utils.getNoNil(mission.completion, 0)*100));found = found+1;
		end;
	end;
	if found > 0 then
		return isNotice, prozent/found;
	end;
	return isNotice, prozent;
end;

moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="onStartMission", callback=moMissions.loadOnStart } );
moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="update", callback=moMissions.update } );