moSellingStations = {};
source(MultiOverlayV4.modDir.."scripte/module/sellingStations/moSellingStationsConfig.lua");
source(MultiOverlayV4.modDir.."scripte/module/sellingStations/moSellingStationsOutput.lua");

function moSellingStations:update(dt)
	if moModule:getActiveRunningModul( {modName="multiOverlayV4", modulName="sellingStations"} ) then
		if moSellingStationsConfig.values.updateTimer > 0 then
			if g_currentMission.hl.timers["updateSellingStations"] ~= nil then return;end;
			g_currentMission.hl.addTimer( {delay=moSellingStationsConfig.values.updateTimer, name="updateSellingStations", repeatable=1, ms=moSellingStationsConfig.values.msec, action=moSellingStations.load} ); 
		else
			moSellingStations:load();
		end;
	end;
end;

function moSellingStations.loadOnStart(update) --onStartMission Callback, update = updateByPlayer !new object or delete object etc. delete your old cmds!	
	if moSellingStationsConfig.values == nil then moSellingStationsConfig:defaultValues();end;	
	if update == nil or not update then moSellingStationsConfig:generateCmd();end;
	moSellingStations:loadAllSellingStations();
	
end;

function moSellingStations.load() --updateTimer Callback	
	moSellingStations:loadAllSellingStations();	
end;

function moSellingStations:loadAllSellingStations()
	local playerFarmId = g_currentMission.player.farmId;	
	--local farmId = g_currentMission:getFarmId();	
	if moModule.sellingStations ~= nil and #moModule.sellingStations > 0 then
		local deleteObject = {};
		for index=1, #moModule.sellingStations do			
			if moModule.sellingStations[index] ~= nil then
				local isObject = moModuleObjects:isObject(moModule.sellingStations[index].object);
				if isObject then
					local sellingStation = moModule.sellingStations[index].object;
					if sellingStation ~= nil and sellingStation.spec_sellingStation ~= nil and sellingStation.spec_sellingStation.sellingStation ~= nil then
						local node = Utils.getNoNil(sellingStation.rootNode, sellingStation.nodeId);
						local mohDisplayName = "Unknown";
						if sellingStation.mohTable ~= nil then 
							mohDisplayName = Utils.getNoNil(sellingStation.mohTable.name, "Unknown");
							sellingStation.mohTable.general={};sellingStation.mohTable.output={};sellingStation.mohTable.input={};
						else							
							sellingStation.mohTable = {general={},output={},input={}};							
						end;
						if sellingStation.mohTable.viewInput == nil then sellingStation.mohTable.viewInput = false;end;
						local spec_sellingStation = sellingStation.spec_sellingStation.sellingStation;
						local stationName = spec_sellingStation.uiName;
						if stationName == nil or type(stationName) ~= "string" or stationName:len() <= 0 then stationName = mohDisplayName;end;
						sellingStation.mohTable.general[#sellingStation.mohTable.general+1] = {};
						local isLineTableGeneral = sellingStation.mohTable.general[#sellingStation.mohTable.general];						
						isLineTableGeneral.objectName = stationName;
						
						local foundVot, isOpening, votOpeningTime, votOpeningTimeString, votSpecialeActive, votSpecialeStop = moModuleSetGet:getVotMod(spec_sellingStation); --VariableOpeningTime Mod Support
						if foundVot then
							isLineTableGeneral.vot = {};
							isLineTableGeneral.vot.openingTime = votOpeningTime;
							isLineTableGeneral.vot.isOpening = isOpening;
							isLineTableGeneral.vot.openingTimeString = votOpeningTimeString;
							isLineTableGeneral.vot.specialeActive = votSpecialeActive;
							isLineTableGeneral.vot.specialeStop = votSpecialeStop;								
						end;
						
						---input---	
						function setInput(fillTypeIndex, price, priceTrend)
							sellingStation.mohTable.input[#sellingStation.mohTable.input+1] = {};
							local isLineTableInput = sellingStation.mohTable.input[#sellingStation.mohTable.input];	
							local fillType = moFillTypes.values.fillTypes[fillTypeIndex];
							if fillType ~= nil then
								isLineTableInput.fillTypeName = fillType.name;
							else
								isLineTableInput.fillTypeName = moLanguageXml:getTxt("unknown", nil, "Unknown");
							end;
							isLineTableInput.fillType = fillTypeIndex;
							isLineTableInput.price = price;
							isLineTableInput.priceTrend = priceTrend;
						end;
						---input---
						
						for fillType,isAccepted in pairs(spec_sellingStation.acceptedFillTypes) do
							if isAccepted and spec_sellingStation:getIsFillTypeAllowed(fillType) then
								local difficultyMultiplier = 1000;
								local price = math.floor(spec_sellingStation:getEffectiveFillTypePrice(fillType) * difficultyMultiplier - 0.5);
								local priceTrend = spec_sellingStation:getCurrentPricingTrend(fillType)
								local isPriceTrend = "normal";
								if priceTrend ~= nil then
									if Utils.isBitSet(priceTrend, SellingStation.PRICE_GREAT_DEMAND) then
										isPriceTrend = "greatDemand";
										isLineTableGeneral.isGreatDemand = true;
									elseif Utils.isBitSet(priceTrend, SellingStation.PRICE_CLIMBING) then
										isPriceTrend = "up";
									elseif Utils.isBitSet(priceTrend, SellingStation.PRICE_FALLING) then
										isPriceTrend = "down";
									end;
								end;
								setInput(fillType, price, isPriceTrend);								
							end;
						end;						
					end;
				end;	
			else
				table.insert(deleteObject, index);
			end;
		end;
		if #deleteObject > 0 then
			for _, index in pairs (deleteObject) do				
				table.remove(moModule.sellingStations, index);
			end;						
		end;				
	end;	
end;

moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="onStartMission", callback=moSellingStations.loadOnStart } );
moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="update", callback=moSellingStations.update } );