moStorages = {};
--source(MultiOverlayV4.modDir.."scripte/module/storages/moStoragesConfig.lua");
--source(MultiOverlayV4.modDir.."scripte/module/storages/moStoragesOutput.lua");

function moStorages:update(dt)
	if moModule:getActiveRunningModul( {modName="multiOverlayV4", modulName="storages"} ) then
		if moStoragesConfig.values.updateTimer > 0 then
			if g_currentMission.hl.timers["updateStorages"] ~= nil then return;end;
			g_currentMission.hl.addTimer( {delay=moStoragesConfig.values.updateTimer, name="updateStorages", repeatable=1, ms=moStoragesConfig.values.msec, action=moStorages.load} ); 
		else
			moStorages:load();
		end;
	end;
end;

function moStorages.loadOnStart(update) --onStartMission Callback, update = updateByPlayer !new object or delete obbject etc. delete your old cmds!	
	if moStoragesConfig.values == nil then moStoragesConfig:defaultValues();end;
	if update then moStoragesConfig:deleteCmd();end;
	moStorages:loadAllStorages(true);	
end;

function moStorages.load() --updateTimer Callback	
	moStorages:loadAllStorages();	
end;

function moStorages:loadAllStorages(generateCmd)	
	local playerFarmId = g_currentMission.player.farmId;	
	--g_i18n:formatVolume(fillLevel)
	if moModule.storages ~= nil and #moModule.storages > 0 then
		local deleteObject = {};
		for index=1, #moModule.storages do			
			if moModule.storages[index] ~= nil then
				local isObject = moModuleObjects:isObject(moModule.storages[index].object);
				if isObject then
					local storage = moModule.storages[index].object;
					if storage.ownerFarmId == 0 or storage.ownerFarmId == playerFarmId and storage.spec_silo ~= nil then						
						if storage.mohTable == nil then
							storage.mohTable = {general={},output={},input={}};						
						else
							storage.mohTable.general={};storage.mohTable.output={};storage.mohTable.input={};
						end;
						if storage.mohTable.viewOutput == nil then storage.mohTable.viewOutput = false;end;	
						if storage.mohTable.viewInput == nil then storage.mohTable.viewInput = false;end;						
						if storage.mohTable.searchExtension == nil then storage.mohTable.searchExtension = false;end;
						if storage.mohTable.siloExtensions == nil then storage.mohTable.siloExtensions = {};end;						
						if not storage.mohTable.searchExtension then
							local isExtension, objects = moStorages:getSiloExtensions(storage);
							storage.mohTable.searchExtension = true;
							if isExtension then storage.mohTable.siloExtensions = objects;end;							
						end;
						---general output---
						--local totalObjectCapacity = moStorages:getCapacity(storage);
						--local fillLevels = moStorages:getFillLevels(storage);
						local fillLevels, totalObjectCapacity = moStorages:getFillLevelInformation(storage);
						storage.mohTable.general[#storage.mohTable.general+1] = {};
						local isLineTableGeneral = storage.mohTable.general[#storage.mohTable.general];												
						if #storage.mohTable.siloExtensions > 0 then
							isLineTableGeneral.objectName = storage.mohTable.name.. " (+".. tostring(#storage.mohTable.siloExtensions).. ")";																		
						else
							isLineTableGeneral.objectName = storage.mohTable.name;	
						end;
						isLineTableGeneral.capacity = 0;
						isLineTableGeneral.capacityTotal = g_i18n:formatNumber(totalObjectCapacity); 
						isLineTableGeneral.capacityName = moLanguageXml:getTxt("capacity", nil, g_i18n:getText("shop_capacity"));
						isLineTableGeneral.capacityColor = 1;						
						isLineTableGeneral.usage = 0; 
						isLineTableGeneral.usageName = moLanguageXml:getTxt("usage", nil, "Usage");
						isLineTableGeneral.usageColor = 1;						
						isLineTableGeneral.age = storage.age;						
						---general output---
						local totalFillLevel = 0;
						local totalCapacity = 0;
						---output---
						function setOutput(fillTypeIndex, fillLevel)
							storage.mohTable.output[#storage.mohTable.output+1] = {};
							local isLineTableOutput = storage.mohTable.output[#storage.mohTable.output];
							local fillType = moFillTypes.values.fillTypes[fillTypeIndex];
							if fillType ~= nil then
								isLineTableOutput.fillTypeName = fillType.name;
							else
								isLineTableOutput.fillTypeName = moLanguageXml:getTxt("unknown", nil, "Unknown");
							end;
							isLineTableOutput.fillLevel = g_i18n:formatNumber(Utils.getNoNil(fillLevel, 0));							
							totalFillLevel = totalFillLevel+Utils.getNoNil(fillLevel, 0);							
						end;
						---output---
						---input---	
						function setInput(fillTypeIndex, fillLevel)
							storage.mohTable.input[#storage.mohTable.input+1] = {};
							local isLineTableInput = storage.mohTable.input[#storage.mohTable.input];	
							local fillType = moFillTypes.values.fillTypes[fillTypeIndex];
							if fillType ~= nil then
								isLineTableInput.fillTypeName = fillType.name;
							else
								isLineTableInput.fillTypeName = moLanguageXml:getTxt("unknown", nil, "Unknown");
							end;							
							if fillLevel > 0 and totalObjectCapacity > 0 then
								isLineTableInput.percentageCapacity = math.floor(100 * fillLevel/totalObjectCapacity);
							else
								isLineTableInput.percentageCapacity = 0;
							end;							
						end;
						---input---	
						if fillLevels ~= nil and #fillLevels > 0 then
							for fillTypeIndex, fillLevel in pairs(fillLevels) do								
								if fillLevel > 0.01 then									
									setOutput(fillTypeIndex, fillLevel);
								end;
								setInput(fillTypeIndex, fillLevel);								
							end;							
						end;						
						if totalFillLevel > 0 and totalObjectCapacity > 0 then
							isLineTableGeneral.usageColor = MultiOverlayV4:get_hl("getProzentColor", {Utils.getNoNil(totalFillLevel/totalObjectCapacity, 0)*100, true}, "getProzentColor Productivity Storages");
							isLineTableGeneral.usage = g_i18n:formatNumber(Utils.getNoNil(totalFillLevel/totalObjectCapacity, 0)*100);
							isLineTableGeneral.capacity = g_i18n:formatNumber(math.max(totalObjectCapacity-totalFillLevel, 0)); --g_i18n:formatNumber(totalObjectCapacity-totalFillLevel);
						end;
						if totalFillLevel == 0 and isLineTableGeneral.capacity == 0 then isLineTableGeneral.capacity = g_i18n:formatNumber(totalObjectCapacity);end;
						if isLineTableGeneral.usageColor == 1 then isLineTableGeneral.usageColor = 2;end;						
						if generateCmd ~= nil and generateCmd then moStoragesConfig:generateCmd(storage);end;						
					end;
				else
					table.insert(deleteObject, index);
				end;
			end;
		end;
		if #deleteObject > 0 then
			for _, index in pairs (deleteObject) do				
				table.remove(moModule.storages, index);
			end;
			moStoragesConfig:deleteCmd(true);			
		end;
	end;
end;

function moStorages:getFreeCapacity(storage, fillType)
	if storage.fillLevels[fillType] == nil then
		return 0;
	end;
	if storage.supportsMultipleFillTypes then
		if storage.capacities[fillType] ~= nil then
			return math.max(storage.capacities[fillType] - storage.fillLevels[fillType], 0);
		else
			local usedCapacity = 0;
			for _, fillLevel in pairs(storage.fillLevels) do
				usedCapacity = usedCapacity + fillLevel;
			end;
			return math.max(storage.capacity - usedCapacity, 0);
		end;
	end;
	local capacity = storage.capacities[fillType] or storage.capacity;
	local usedCapacity = 0;
	for usedFillType, usedFillLevel in pairs(storage.fillLevels) do
		if fillType == usedFillType then
			usedCapacity = usedFillLevel;
		elseif usedFillLevel > 0 then
			return 0;
		end;
	end;
	return math.max(capacity - usedCapacity, 0);
end;

function moStorages:getFillLevels(storage)
	local fillLevels = {};
	local storageOut = storage.spec_silo.storages;
	fillLevels = storage:getFillLevels();
	if storageOut ~= nil and storageOut[1] ~= nil and storageOut[1].loadingStations ~= nil then	
		for key, value in pairs (storageOut[1].loadingStations) do
			if storageOut[1].loadingStations[value] ~= nil and (storage.spec_silo.loadingStation ~= value or #fillLevels <= 0) then
				for _, sourceStorage in pairs(storageOut[1].loadingStations[value].sourceStorages) do		
					for fillType, fillLevel in pairs(sourceStorage:getFillLevels()) do
						fillLevels[fillType] = Utils.getNoNil(fillLevels[fillType], 0) + fillLevel
					end;		
				end;
			end;
		end;
	end;	
	return fillLevels;
end;

function moStorages:getCapacity(storage)
	local deleteObject = {};
	local capacity = 0;
	local storageOut = storage.spec_silo.storages;
	if storageOut ~= nil and storageOut[1] ~= nil then
		capacity = Utils.getNoNil(storageOut[1].capacity, 0)
	end;
	if storage.mohTable.siloExtensions ~= nil and #storage.mohTable.siloExtensions > 0 then
		for s=1, #storage.mohTable.siloExtensions do
			local object = moModuleObjects:getObject(storage.mohTable.siloExtensions[s], "storageExtensions");
			local isObject = moModuleObjects:isObject(object);
			if isObject then
				if object.spec_siloExtension ~= nil and object.spec_siloExtension.storage ~= nil then
					capacity = capacity + Utils.getNoNil(object.spec_siloExtension.storage.capacity, 0);
				end;
			end;
		end;	
	end;		
	if #deleteObject > 0 then
		for _, index in pairs (deleteObject) do				
			table.remove(storage.mohTable.siloExtensions, index);
		end;					
	end;
	return capacity;
end;

function moStorages:getFillLevelInformation(storage, isTrainStorage, isExtension)	
	local deleteObject = {};
	local fillLevelInformation = {};
	local fillLevelInformationE = {};
	local capacity = 0;	
	if storage.spec_silo ~= nil then
		capacity = storage.spec_silo.storages[1].capacity;
		fillLevelInformation = storage:getFillLevels();
	end;
	if storage.mohTable.siloExtensions ~= nil and #storage.mohTable.siloExtensions > 0 then
		for s=1, #storage.mohTable.siloExtensions do
			local object = moModuleObjects:getObject(storage.mohTable.siloExtensions[s], "storageExtensions");
			local isObject = moModuleObjects:isObject(object);
			if isObject then
				if object.spec_siloExtension ~= nil and object.spec_siloExtension.storage ~= nil and object.spec_siloExtension.storage.fillLevels ~= nil then
					capacity = capacity + object.spec_siloExtension.storage.capacity;
					for fillTypeIndex, fillLevel in pairs(object.spec_siloExtension.storage.fillLevels) do
						fillLevelInformation[fillTypeIndex] = Utils.getNoNil(fillLevelInformation[fillTypeIndex], 0) + fillLevel;						
					end;
				end;
			else
				table.insert(deleteObject, s);
			end;
		end;
	end;	
	if #deleteObject > 0 then
		for _, index in pairs (deleteObject) do				
			table.remove(storage.mohTable.siloExtensions, index);
		end;					
	end;
	return fillLevelInformation, capacity;
end;

function moStorages:getSiloExtensions(storage)
	if moModule.storageExtensions == nil or #moModule.storageExtensions <= 0 then return false;end;
	local storageExtensions = {};
	function isAlReadyExists(rootNode)
		for a=1, #storageExtensions do
			if storageExtensions[a] == rootNode then return true;end;
		end;
		return false;
	end;
	function isObjectFree(rootNode)
		for a=1, #moModule.storageExtensionsFree do
			if moModule.storageExtensionsFree[a] == rootNode then return false;end;
		end;
		return true;
	end;
	for s=1, #moModule.storageExtensions do
		local object = moModule.storageExtensions[s].object;
		if object.spec_siloExtension ~= nil and object.spec_siloExtension.storage ~= nil then
			if object.spec_siloExtension.storage.loadingStations ~= nil then				
				for key, value in pairs (object.spec_siloExtension.storage.loadingStations) do					
					if storage.spec_silo.storages[1].loadingStations[value] ~= nil then					
						if not isAlReadyExists(object.rootNode) and isObjectFree(object.rootNode) then
							table.insert(storageExtensions, object.rootNode);
							table.insert(moModule.storageExtensionsFree, object.rootNode);
						end;
					end;
				end;
			end;
		end;
	end;	
	return #storageExtensions > 0, storageExtensions;
end;

--moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="onStartMission", callback=moStorages.loadOnStart } );
--moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="update", callback=moStorages.update } );