moWindTurbinesConfig = {};

function moWindTurbinesConfig:defaultValues()
	moWindTurbinesConfig.values = {updateTimer=500, msec=false};
end;

function moWindTurbinesConfig.loadXml(Xml, xmlNameTag, directory, typ) --loadConfig Callback	
	if typ ~= "profilSettings" then return;end;	
	if moWindTurbinesConfig.values == nil then moWindTurbinesConfig:defaultValues();end;
	local groupNameTag = (xmlNameTag.. ".module.windTurbines(%d)"):format(0);
	if getXMLBool(Xml, groupNameTag.. "#mSecTimer") ~= nil then
		moWindTurbinesConfig.values.msec = getXMLBool(Xml, groupNameTag.. "#mSecTimer");
	else
		return; --first config not found
	end;
	if getXMLInt(Xml, groupNameTag.. "#updateTimer") ~= nil then
		moWindTurbinesConfig.values.updateTimer = getXMLInt(Xml, groupNameTag.. "#updateTimer");
	end;	
end;

function moWindTurbinesConfig.saveXml(Xml, xmlNameTag, directory, typ) --saveConfig Callback
	if typ ~= "profilSettings" then return;end;	
	if moWindTurbinesConfig.values == nil then moWindTurbinesConfig:defaultValues();end;
	----------------------------------------------------------------------------------------------
	local groupNameTag = (xmlNameTag.. ".module.windTurbines(%d)"):format(0);
	setXMLInt(Xml, groupNameTag.. "#updateTimer", moWindTurbinesConfig.values.updateTimer);
	setXMLBool(Xml, groupNameTag.. "#mSecTimer", moWindTurbinesConfig.values.msec);
	----------------------------------------------------------------------------------------------	
end;

function moWindTurbinesConfig:refreshCmdDisplayName(cmdRegName, displayName) --ls22 edit object name
	if MultiOverlayV4.cmd[cmdRegName] ~= nil then
		MultiOverlayV4.cmd[cmdRegName].displayName = tostring(displayName);
		MultiOverlayV4.cmd[cmdRegName].infoTxt = "View WindTurbine (".. tostring(displayName).. ")";
		local slots = moSetGetSlot:getSlots();
		if slots ~= nil and #slots > 0 then
			for slot=1, #slots do
				if moSetGetOverlays.slot[slots[slot]].cmd ~= nil and #moSetGetOverlays.slot[slots[slot]].cmd > 0 then
					for cmd=1, #moSetGetOverlays.slot[slots[slot]].cmd do
						if moSetGetOverlays.slot[slots[slot]].cmd[cmd].regName == cmdRegName then
							moSetGetOverlays.slot[slots[slot]].cmd[cmd].displayName = tostring(displayName);							
						end;
					end;
				end;
			end;
		end;
	end;
end;

function moWindTurbinesConfig:generateAllCmd()
	function generateCmd(ownTable)
		moSetGetCmd:generateCmd( {	
				modName = "MultiOverlayV4";
				cmdName = "output_allWindturbines_";
				groupName = "WindTurbines";			
				displayName = "View All WindTurbines";
				infoTxt = "View All WindTurbines";
				typ = {"output_"};
				mouseClickAccepts = {true,true,true};
				ownTable = ownTable;
				object = {};
				actionCallback = moWindTurbinesOutput.giveAllOutputTable;			
			}
		)
	end;
	generateCmd({});
end;


function moWindTurbinesConfig:generateCmd(object)
	function generateCmd(cmdName, displayName, object, ownTable)
		moSetGetCmd:generateCmd( {	
				modName = "MultiOverlayV4";
				cmdName = cmdName;
				groupName = "WindTurbines";			
				displayName = tostring(displayName);
				infoTxt = "View WindTurbine (".. tostring(displayName).. ")";
				typ = {"output_"};
				mouseClickAccepts = {true,true,true};
				ownTable = ownTable;
				object = {object};
				actionCallback = moWindTurbinesOutput.giveOutputTable;			
			}
		)
	end;
	if object ~= nil and moModuleObjects:isObject(object) then
		local objectId = object.nodeId or object.rootNode;		
		local x, y ,z = getWorldTranslation(objectId);
		local cmdName = "";
		local displayName = object.mohTable.name;
		local tempName = string.gsub(object.mohTable.name, " ", "");
		if object.mohTable.mohDisplayName ~= nil then displayName = object.mohTable.mohDisplayName;end;
		if x == nil or y == nil or z == nil then			
			cmdName = "output_windturbine_".. tostring(tempName).. "_";
		else
			cmdName = "output_windturbine_".. tostring(tempName).. "_x"..tostring(x).."_y"..tostring(y).."_z"..tostring(z).."_";
		end;
		if MultiOverlayV4.cmd["MultiOverlayV4_cmd_"..cmdName] ~= nil and MultiOverlayV4.cmd["MultiOverlayV4_cmd_"..cmdName].object ~= nil then
			MultiOverlayV4.cmd["MultiOverlayV4_cmd_"..cmdName].object[#MultiOverlayV4.cmd["MultiOverlayV4_cmd_"..cmdName].object+1] = object;
		else
			generateCmd(cmdName, displayName, object, {});
		end;
	end;
end;

function moWindTurbinesConfig:deleteCmd(deleteObject)
	local groupCmdTable = moSetGetCmd:getGroupCmdTable("WindTurbines");
	if groupCmdTable ~= nil then
		for a=1, #groupCmdTable do
			if groupCmdTable[a] ~= nil and groupCmdTable[a].cmd ~= nil then
				if string.find(groupCmdTable[a].cmd, "MultiOverlayV4_cmd_output_windturbine_") then
					local subCmd = string.gsub(groupCmdTable[a].cmd, "MultiOverlayV4_cmd_", "");
					if deleteObject ~= nil and MultiOverlayV4.cmd[groupCmdTable[a].cmd] ~= nil and MultiOverlayV4.cmd[groupCmdTable[a].cmd].object ~= nil then
						for o=1, #MultiOverlayV4.cmd[groupCmdTable[a].cmd].object do
							if MultiOverlayV4.cmd[groupCmdTable[a].cmd].object[o] ~= nil then
								local isObject = moModuleObjects:isObject(MultiOverlayV4.cmd[groupCmdTable[a].cmd].object[o]);
								if isObject == nil or not isObject then 
									if #MultiOverlayV4.cmd[groupCmdTable[a].cmd].object == 1 then
										moSetGetCmd:deleteCmd( {modName="MultiOverlayV4", cmdName=subCmd} );
										if moWindTurbines[groupCmdTable[a].cmd] ~= nil then moWindTurbines[groupCmdTable[a].cmd] = nil;end;
									else
										MultiOverlayV4.cmd[groupCmdTable[a].cmd].object[o] = nil;
									end;
								end;
							end;
						end;
					else					
						moSetGetCmd:deleteCmd( {modName="MultiOverlayV4", cmdName=subCmd} );
					end;
				end;
			end;
		end;
	end;
end;

moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="loadConfig", callback=moWindTurbinesConfig.loadXml } );
moModule:addCallbackModul( {modName="multiOverlayV4", modulCallbackName="saveConfig", callback=moWindTurbinesConfig.saveXml } );
